#include <getopt.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

#include "parsing.h"

char *get_option_arg(struct option_lst *options, e_flag flag)
{
	if (!options)
		return 0;
	for (struct option_lst *it = options; it; it = it->next)
		if (it->flag == flag)
			return it->arg;
	return 0;
}

static int add_option(struct option_lst **head, e_flag flag, char *arg)
{
	struct option_lst *new_option = malloc(sizeof(struct option_lst));
	if (!new_option) {
		dprintf(2, "ft_ping: allocation of option failed\n");
		for (struct option_lst *it = *head; it;) {
			struct option_lst *tmp = it;
			it = it->next;
			free(tmp);
		}
		return -1;
	}

	new_option->flag = flag;
	new_option->arg = arg;
	new_option->next = NULL;

	if (*head == NULL) {
		*head = new_option;
		return 0;
	}
	for (struct option_lst *it = *head; it; it = it->next) {
		if (it->next == NULL) {
			it->next = new_option;
			break;
		}
	}
	return 0;
}

struct option_lst *parse_options(int ac, char *const *av)
{
	struct option_lst *head = NULL;
	const struct option options[NB_OPTIONS] = {
	    {"help", no_argument, 0, 0},
	    {"ports", required_argument, 0, 0},
	    {"ip", required_argument, 0, 0},
	    {"speedup", required_argument, 0, 0},
	    {"scan", required_argument, 0, 0},
	};

	int c;
	while (1) {
		int option_index = 0;
		c = getopt_long(ac, av, "", options, &option_index);
		if (c == -1)
			break;
		else if (c)
			return NULL;
		if (add_option(&head, option_index, optarg) < 0)
			return NULL;
	}
	return head;
}
